package com.indicee.customtabledemo.client;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

import com.google.gwt.core.client.EntryPoint;
import com.google.gwt.core.client.GWT;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Button;
import com.google.gwt.user.client.ui.FlexTable;
import com.google.gwt.user.client.ui.InlineLabel;
import com.google.gwt.user.client.ui.ListBox;
import com.google.gwt.user.client.ui.RootPanel;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.data.DataRow;
import com.indicee.api.objects.report.view.filter.FieldFilter;
import com.indicee.api.objects.report.view.filter.FieldFilterValue;
import com.indicee.api.objects.report.view.filter.Filter;

/**
 * Entry point classes define <code>onModuleLoad()</code>.
 */
public class CustomTableDemo implements EntryPoint {
    /**
     * Create a remote service proxy to talk to the server-side Greeting
     * service.
     */
    private final IndiceeDataServiceAsync indiceeDataService = GWT.create(IndiceeDataService.class);

    private final FlexTable jobs = new FlexTable();

    private ListBox companySelectBox;

    private ListBox divisionSelectBox;

    private FlexTable headerTable;

    private ListBox monthSelectBox;

    private ListBox yearSelectBox;

    private final HashMap<String, Integer> jobIndex = new HashMap<String, Integer>();

    private final int REPORT_ID = 1036;

    private Button regenerateReportButton;

    /**
     * This is the entry point method.
     */
    public void onModuleLoad() {

        monthSelectBox = new ListBox();
        monthSelectBox.addItem("Jan");
        monthSelectBox.addItem("Feb");
        monthSelectBox.addItem("Mar");
        monthSelectBox.addItem("Apr");
        monthSelectBox.addItem("May");
        monthSelectBox.addItem("Jun");
        monthSelectBox.addItem("Jul");
        monthSelectBox.addItem("Aug");
        monthSelectBox.addItem("Sep");
        monthSelectBox.addItem("Oct");
        monthSelectBox.addItem("Nov");
        monthSelectBox.addItem("Dec");
        monthSelectBox.setSelectedIndex(7);
        RootPanel.get("monthSelect").add(monthSelectBox);

        yearSelectBox = new ListBox();
        yearSelectBox.addItem("2006");
        yearSelectBox.addItem("2007");
        yearSelectBox.addItem("2008");
        yearSelectBox.addItem("2009");
        yearSelectBox.addItem("2010");
        yearSelectBox.setSelectedIndex(4);
        RootPanel.get("yearSelect").add(yearSelectBox);

        companySelectBox = new ListBox();
        companySelectBox.addItem("Company A", "5");
        companySelectBox.addItem("Company B", "9");
        companySelectBox.addItem("Company C", "1");
        companySelectBox.addItem("Company D", "8");
        companySelectBox.setSelectedIndex(2);
        companySelectBox.addChangeHandler(new ChangeHandler() {

            @Override
            public void onChange(ChangeEvent event) {
                populateDivisionSelectBox();
            }
        });

        RootPanel.get("companySelect").add(companySelectBox);

        divisionSelectBox = new ListBox();
        populateDivisionSelectBox();
        divisionSelectBox.setSelectedIndex(1);

        RootPanel.get("divisionSelect").add(divisionSelectBox);

        regenerateReportButton = new Button("Regenerate Report");
        regenerateReportButton.addClickHandler(new ClickHandler() {

            @Override
            public void onClick(ClickEvent event) {
                regenerateReport();
            }
        });

        RootPanel.get("regenerateReport").add(regenerateReportButton);

        int columnNum = 0;
        headerTable = new FlexTable();

        headerTable.setText(0, columnNum++, "Job #");
        headerTable.setText(0, columnNum++, "Job Location");
        columnNum++;
        columnNum++;
        headerTable.setText(0, columnNum++, "Revenue");
        headerTable.setText(0, columnNum++, "Labour");
        headerTable.setText(0, columnNum++, "Material");
        headerTable.setText(0, columnNum++, "Interco Equip");
        headerTable.setText(0, columnNum++, "Cartage");
        headerTable.setText(0, columnNum++, "Sub-Contract");
        headerTable.setText(0, columnNum++, "Dump Fees");
        headerTable.setText(0, columnNum++, "Outside Equip");
        headerTable.setText(0, columnNum++, "Other Costs");
        headerTable.setText(0, columnNum++, "Total Expenses");
        headerTable.setText(0, columnNum++, "Profit");
        headerTable.setText(0, columnNum++, "Margin %");

        columnNum = 1;
        headerTable.setText(1, columnNum++, "Expected %");
        columnNum++;
        columnNum++;
        headerTable.setText(1, columnNum++, "100%");
        headerTable.setText(1, columnNum++, "40.0%");
        headerTable.setText(1, columnNum++, "4.5%");
        headerTable.setText(1, columnNum++, "7.%");
        headerTable.setText(1, columnNum++, "3.0%");
        headerTable.setText(1, columnNum++, "12.0%");
        headerTable.setText(1, columnNum++, "5.0%");
        headerTable.setText(1, columnNum++, "11.0%");
        headerTable.setText(1, columnNum++, "2.0%");
        headerTable.setText(1, columnNum++, "84.7%");
        headerTable.setText(1, columnNum++, "15.3%");

        headerTable.addStyleName("headerTable");
        headerTable.getRowFormatter().addStyleName(0, "headers");
        headerTable.setCellSpacing(0);
        headerTable.setCellPadding(0);
        for (int i = 0; i < headerTable.getRowCount(); i++) {
            for (int j = 0; j < headerTable.getCellCount(i); j++) {
                headerTable.getCellFormatter().addStyleName(i, j, "column-" + j);
                headerTable.getCellFormatter().addStyleName(i, j, "row-" + i);
                headerTable.getCellFormatter().addStyleName(i, j, "column");
            }
        }

        jobs.setWidget(0, 0, headerTable);

        jobs.addStyleName("jobsTable");
        jobs.setCellSpacing(0);
        jobs.setCellPadding(0);

        //regenerateReport();

        RootPanel.get("dataTable").add(jobs);
    }

    protected void populateDivisionSelectBox() {
        divisionSelectBox.clear();
        switch (companySelectBox.getSelectedIndex()) {
        case 0:
            divisionSelectBox.addItem("Division A", "66");
            divisionSelectBox.addItem("Division B", "67");
            divisionSelectBox.addItem("Division C", "06");
            divisionSelectBox.addItem("Division D", "08");
            break;
        case 1:
            divisionSelectBox.addItem("Division E", "88");
            break;
        case 2:
            divisionSelectBox.addItem("Division F", "09");
            divisionSelectBox.addItem("Division G", "07");
            divisionSelectBox.addItem("Division H", "08");
            break;
        case 3:
            divisionSelectBox.addItem("Division I", "55");
            divisionSelectBox.addItem("Division J", "44");
            break;
        }
    }

    protected void regenerateReport() {
        disableControls();
        jobs.clear();
        regenerateReportJTD();
    }

    private void disableControls() {
        companySelectBox.setEnabled(false);
        divisionSelectBox.setEnabled(false);
        monthSelectBox.setEnabled(false);
        yearSelectBox.setEnabled(false);
        regenerateReportButton.setEnabled(false);
    }

    protected void regenerateReportCurMonth() {
        RootPanel.get("loading").clear();
        RootPanel.get("loading").add(new InlineLabel("loading Month ..."));
        List<Filter> fieldFilters = new ArrayList<Filter>();

        FieldFilter filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/fields/Company");
        filter.setInclusive(true);
        List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
        values.add(new FieldFilterValue("\"" + companySelectBox.getValue(companySelectBox.getSelectedIndex()) + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/fields/Div");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        values
                .add(new FieldFilterValue("\"" + divisionSelectBox.getValue(divisionSelectBox.getSelectedIndex())
                        + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/entities/PerHierarchy/Month_name");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        values.add(new FieldFilterValue("\"" + monthSelectBox.getValue(monthSelectBox.getSelectedIndex()) + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/entities/PerHierarchy/Year_name");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        values.add(new FieldFilterValue("\"" + yearSelectBox.getValue(yearSelectBox.getSelectedIndex()) + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        indiceeDataService.getReport(REPORT_ID, fieldFilters, new AsyncCallback<Report>() {

            @Override
            public void onSuccess(Report reportData) {
                displayReportData(reportData, 0);
                RootPanel.get("loading").clear();
                RootPanel.get("loading").add(new InlineLabel("Complete."));
                enableControls();
            }

            @Override
            public void onFailure(Throwable caught) {
                RootPanel.get("loading").clear();
                RootPanel.get("loading").add(new InlineLabel("Failed: " + caught.getMessage()));
                enableControls();
            }
        });
    }

    protected void regenerateReportYTD() {
        RootPanel.get("loading").clear();
        RootPanel.get("loading").add(new InlineLabel("loading Y/T/D ..."));
        List<Filter> fieldFilters = new ArrayList<Filter>();

        FieldFilter filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/fields/Company");
        filter.setInclusive(true);
        List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
        values.add(new FieldFilterValue("\"" + companySelectBox.getValue(companySelectBox.getSelectedIndex()) + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/fields/Div");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        values
                .add(new FieldFilterValue("\"" + divisionSelectBox.getValue(divisionSelectBox.getSelectedIndex())
                        + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/entities/PerHierarchy/Month_name");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        for (int i = 0; i < monthSelectBox.getSelectedIndex() + 1; i++) {
            values.add(new FieldFilterValue("\"" + monthSelectBox.getValue(i) + "\""));
        }
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/entities/PerHierarchy/Year_name");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        values.add(new FieldFilterValue("\"" + yearSelectBox.getValue(yearSelectBox.getSelectedIndex()) + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        indiceeDataService.getReport(REPORT_ID, fieldFilters, new AsyncCallback<Report>() {

            @Override
            public void onSuccess(Report reportData) {
                displayReportData(reportData, 1);
                regenerateReportCurMonth();
            }

            @Override
            public void onFailure(Throwable caught) {
                RootPanel.get("loading").clear();
                RootPanel.get("loading").add(new InlineLabel("Failed: " + caught.getMessage()));
                enableControls();
            }
        });
    }

    protected void regenerateReportJTD() {
        RootPanel.get("loading").clear();
        RootPanel.get("loading").add(new InlineLabel("loading J/T/D ..."));
        List<Filter> fieldFilters = new ArrayList<Filter>();

        FieldFilter filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/fields/Company");
        filter.setInclusive(true);
        List<FieldFilterValue> values = new ArrayList<FieldFilterValue>();
        values.add(new FieldFilterValue("\"" + companySelectBox.getValue(companySelectBox.getSelectedIndex()) + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/fields/Div");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        values
                .add(new FieldFilterValue("\"" + divisionSelectBox.getValue(divisionSelectBox.getSelectedIndex())
                        + "\""));
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/entities/PerHierarchy/Month_name");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        for (int i = 0; i < monthSelectBox.getSelectedIndex() + 1; i++) {
            values.add(new FieldFilterValue("\"" + monthSelectBox.getValue(i) + "\""));
        }
        filter.setValues(values);
        fieldFilters.add(filter);

        filter = new FieldFilter();
        filter.setField("iqn.1://indicee.com/79/dataModel/entities/PerHierarchy/Year_name");
        filter.setInclusive(true);
        values = new ArrayList<FieldFilterValue>();
        for (int i = 0; i < yearSelectBox.getSelectedIndex() + 1; i++) {
            values.add(new FieldFilterValue("\"" + yearSelectBox.getValue(i) + "\""));
        }
        filter.setValues(values);
        fieldFilters.add(filter);

        indiceeDataService.getReport(REPORT_ID, fieldFilters, new AsyncCallback<Report>() {

            @Override
            public void onSuccess(Report reportData) {
                displayReportData(reportData, 2);
                regenerateReportYTD();
            }

            @Override
            public void onFailure(Throwable caught) {
                RootPanel.get("loading").clear();
                RootPanel.get("loading").add(new InlineLabel("Failed: " + caught.getMessage()));
                enableControls();
            }
        });
    }

    protected void enableControls() {
        companySelectBox.setEnabled(true);
        divisionSelectBox.setEnabled(true);
        monthSelectBox.setEnabled(true);
        yearSelectBox.setEnabled(true);
        regenerateReportButton.setEnabled(true);
    }

    protected void displayReportData(Report reportData, int type) {
        jobs.setWidget(0, 0, headerTable);
        String period = "";
        switch (type) {
        case 0:
            period = "Month";
            break;
        case 1:
            period = "Y/T/D";
            break;
        case 2:
            period = "J/T/D";
            buildJobIndex(reportData);
            break;
        }

        List<DataRow> rows = reportData.getReportData().getReportDataRows();

        for (int j = 1; j < rows.size(); j++) {
            DataRow row = rows.get(j);

            int index = -1;
            if (row.getCells().get(1).equals("Total")) {
                index = jobIndex.get(row.getCells().get(0)) + 2;
            } else {
                index = jobIndex.get(row.getCells().get(1)) + 2;
            }
            FlexTable jobTable = jobs.isCellPresent(index, 0) ? (FlexTable) jobs.getWidget(index, 0) : null;
            double revenue = 0;
            if (row.getCells().get(5) != null) {
                revenue = Double.parseDouble(row.getCells().get(5).replaceAll(",", ""));
            }
            if (jobTable == null) {
                jobTable = new FlexTable();
                if (row.getCells().get(1).equals("Total")) {
                    if (row.getCells().get(0).equals("Total")) {
                        jobTable.setText(0, 1, "NET DIVISIONAL PROFIT");
                        jobTable.getFlexCellFormatter().addStyleName(0, 1, "bold");
                    } else if (row.getCells().get(0).equals("Operations")) {
                        jobTable.setText(0, 1, "GROSS PROFIT CONTR.");
                        jobTable.getFlexCellFormatter().addStyleName(0, 1, "bold");
                    } else if (row.getCells().get(0).equals("Overhead")) {
                        continue;
                    }
                } else {
                    jobTable.setText(0, 0, row.getCells().get(1));
                    jobTable.setText(0, 1, row.getCells().get(2));
                    if (!row.getCells().get(0).equals("Overhead")) {

                        jobTable.setText(1, 0, "ST");
                        jobTable.setText(1, 1,
                                (row.getCells().get(4) != null && row.getCells().get(4).length() >= 10) ? row
                                        .getCells().get(4).substring(0, 10) : null);
                        jobTable.setText(2, 0, "AMT");
                        jobTable.setText(2, 1, row.getCells().get(3));
                        double amount = -1;
                        if (row.getCells().get(3) != null) {
                            amount = Double.parseDouble(row.getCells().get(3).replaceAll(",", ""));
                        }

                        if (revenue != 0 && amount != -1) {
                            int percentComplete = (int) (amount / revenue * 100);
                            jobTable.setText(2, 2, String.valueOf(percentComplete) + "%");
                        } else {
                            jobTable.setText(2, 2, "N/A");
                        }
                    }
                }
                jobs.setWidget(index, 0, jobTable);
                jobTable.setStyleName("itemTable");
                jobTable.setCellSpacing(0);
                jobTable.setCellPadding(0);
                jobTable.setText(0, 15, "");
                jobTable.setText(1, 15, "");
                jobTable.setText(2, 15, "");
                for (int i = 0; i < jobTable.getRowCount(); i++) {
                    for (int k = 0; k < jobTable.getCellCount(i); k++) {
                        jobTable.getCellFormatter().addStyleName(i, k, "column-" + k);
                        jobTable.getCellFormatter().addStyleName(i, k, "row-" + i);
                        jobTable.getCellFormatter().addStyleName(i, k, "column");
                    }
                }
            }

            jobTable.setText(type, 3, period);
            for (int i = 5; i < row.getCells().size(); i++) {
                jobTable.setText(type, i - 1, row.getCells().get(i) != null ? row.getCells().get(i) : "0");
            }
            if (revenue == 0) {
                jobTable.setText(type, 15, "N/A");
            }
        }
    }

    private void buildJobIndex(Report reportData) {
        List<DataRow> rows = reportData.getReportData().getReportDataRows();
        for (int i = 1; i < rows.size(); i++) {
            if (rows.get(i).getCells().get(1).equals("Total")) {
                jobIndex.put(rows.get(i).getCells().get(0), i);
            } else {
                jobIndex.put(rows.get(i).getCells().get(1), i);
            }
        }
    }
}
