package com.indicee.api.objects.report;

import java.io.Serializable;
import java.util.Date;
import java.util.Set;

import com.indicee.api.objects.report.data.Data;
import com.indicee.api.objects.report.view.View;

public class Report extends IndiceeObject implements Serializable {

    private static final long serialVersionUID = 2970302563525595882L;
    private int id;
    private String name;
    private int ownerID;
    private boolean deleted;
    private Date createdDate;
    private Set<Metadata> measures;
    private Set<Metadata> groups;
    private VisualizationOptions visualizationOptions;
    private View reportView;
    private Data reportData;

    public void setId(int id) {
        this.id = id;
    }

    public int getId() {
        return id;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getName() {
        return name;
    }

    public void setDeleted(boolean deleted) {
        this.deleted = deleted;
    }

    public boolean isDeleted() {
        return deleted;
    }

    public void setOwnerID(int ownerID) {
        this.ownerID = ownerID;
    }

    public int getOwnerID() {
        return ownerID;
    }

    public void setCreatedDate(Date createdDate) {
        this.createdDate = createdDate;
    }

    public Date getCreatedDate() {
        return createdDate;
    }

    public void setMeasures(Set<Metadata> measures) {
        this.measures = measures;
    }

    public Set<Metadata> getMeasures() {
        return measures;
    }

    public void setGroups(Set<Metadata> groups) {
        this.groups = groups;
    }

    public Set<Metadata> getGroups() {
        return groups;
    }

    public void setVisualizationOptions(VisualizationOptions visualizationOptions) {
        this.visualizationOptions = visualizationOptions;
    }

    public VisualizationOptions getVisualizationOptions() {
        return visualizationOptions;
    }

    public void setReportView(View reportView) {
        this.reportView = reportView;
    }

    public View getReportView() {
        return reportView;
    }

    public void setReportData(Data reportData) {
        this.reportData = reportData;
    }

    public Data getReportData() {
        return reportData;
    }

    @Override
    public String toString() {
        return toString("", "");
    }

    public String toFormatedString() {
        return toString("\n", "\t");
    }

    private String toString(String newLine, String tab) {
        StringBuilder sb = new StringBuilder();
        sb.append("<report>").append(newLine);
        sb.append(tab).append("<id>").append(id).append("</id>").append(newLine);
        sb.append(tab).append("<name>").append(name).append("</name>").append(newLine);
        sb.append(tab).append("<owner_id>").append(ownerID).append("</owner_id>").append(newLine);
        sb.append(tab).append("<deleted>").append(deleted).append("</deleted>").append(newLine);
        sb.append(tab).append("<created_datetime>").append(createdDate != null ? dateToString(createdDate) : "")
                .append("</created_datetime>").append(newLine);

        sb.append(tab).append("<metadata>").append(newLine);
        sb.append(tab + tab).append("<measures>").append(newLine);
        if (measures != null) {
            for (Metadata measure : measures) {
                sb.append(tab + tab + tab).append(measure).append(newLine);
            }
        }
        sb.append(tab + tab).append("</measures>").append(newLine);
        sb.append(tab + tab).append("<groups>").append(newLine);
        if (groups != null) {
            for (Metadata group : groups) {
                sb.append(tab + tab + tab).append(group).append(newLine);
            }
        }
        sb.append(tab + tab).append("</groups>").append(newLine);
        sb.append(tab).append("</metadata>").append(newLine);

        sb.append(tab).append("<vis>").append(newLine);
        sb.append(tab).append("</vis>").append(newLine);
        if (reportView != null) {
            sb.append(reportView.toString(newLine, tab, tab)).append(newLine);
        }
        if (reportData != null) {
            sb.append(reportData.toString(newLine, tab, tab)).append(newLine);
        }

        sb.append("</report>").append(newLine);
        return sb.toString();
    }
}
