package com.indicee.api.objects.report.view;

import java.io.Serializable;
import java.util.List;

import com.indicee.api.objects.report.Collection;
import com.indicee.api.objects.report.view.attributes.AttributesOrigin;
import com.indicee.api.objects.report.view.axes.Axis;
import com.indicee.api.objects.report.view.filter.Filter;

public class View implements Serializable {

    private static final long serialVersionUID = 4844209819566270771L;
    private List<Axis> reportViewAxes;
    private List<Filter> fieldFilters;
    private List<Collection> collections;
    private List<AttributesOrigin> attributeOrigins;

    public List<AttributesOrigin> getAttributeOrigins() {
        return attributeOrigins;
    }

    public void setAttributeOrigins(List<AttributesOrigin> attributeOrigins) {
        this.attributeOrigins = attributeOrigins;
    }

    public List<Collection> getCollections() {
        return collections;
    }

    public List<Filter> getFieldFilters() {
        return fieldFilters;
    }

    public List<Axis> getReportViewAxes() {
        return reportViewAxes;
    }

    public void setReportViewAxes(List<Axis> reportViewAxes) {
        this.reportViewAxes = reportViewAxes;
    }

    public void setFieldFilters(List<Filter> fieldFilters) {
        this.fieldFilters = fieldFilters;
    }

    public void setCollections(List<Collection> collections) {
        this.collections = collections;
    }

    @Override
    public String toString() {
        return toString("", "");
    }

    public String toFormatedString() {
        return toString("\n", "\t");
    }

    private String toString(String newLine, String tab) {
        return toString(newLine, tab, "");
    }

    public String toString(String newLine, String tab, String indent) {
        StringBuilder sb = new StringBuilder();
        sb.append(indent).append("<view>").append(newLine);
        sb.append(indent).append(tab).append("<axes>").append(newLine);
        for (Axis axis : reportViewAxes) {
            sb.append(axis.toString(newLine, tab, indent + tab + tab)).append(newLine);
        }
        sb.append(indent).append(tab).append("</axes>").append(newLine);

        sb.append(indent).append(tab).append("<fieldfilters>").append(newLine);
        for (Filter filter : fieldFilters) {
            sb.append(filter.toString(newLine, tab, indent + tab + tab)).append(newLine);
        }
        sb.append(indent).append(tab).append("</fieldfilters>").append(newLine);

        sb.append(indent).append(tab).append("<collections>").append(newLine);
        if (collections != null && !collections.isEmpty()) {
            for (Collection collection : collections) {
                sb.append(collection.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        }
        sb.append(indent).append(tab).append("</collections>").append(newLine);

        sb.append(indent).append(tab).append("<attributes>").append(newLine);
        if (attributeOrigins != null && !attributeOrigins.isEmpty()) {
            for (AttributesOrigin attributeOrigin : attributeOrigins) {
                sb.append(attributeOrigin.toString(newLine, tab, indent + tab + tab)).append(newLine);
            }
        }
        sb.append(indent).append(tab).append("</attributes>").append(newLine);
        sb.append(indent).append("</view>");
        return sb.toString();
    }

}
