package com.indicee.api.parser;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.indicee.api.objects.contribution.Contribution;

public final class ContributionParser extends IndiceeParser {

    private enum ContributionElement {
        ID("id"), UniqueName("unique_name"), OwnerID("owner_id"), Size("size"), Status("status");

        public static ContributionElement get(String name) {
            for (ContributionElement item : ContributionElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private ContributionElement(String value) {
            this.value = value;
        }
    }

    public static Contribution createContribution(Document contributionDocument) {
        Contribution contribution = new Contribution();
        for (int i = 0; i < contributionDocument.getDocumentElement().getChildNodes().getLength(); i++) {
            Node child = contributionDocument.getDocumentElement().getChildNodes().item(i);
            ContributionElement elem = ContributionElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ID:
                    contribution.setId(getInteger(child));
                    break;
                case UniqueName:
                    contribution.setUniqueName(getString(child));
                    break;
                case OwnerID:
                    contribution.setOwnerID(getInteger(child));
                    break;
                case Size:
                    contribution.setSize(getInteger(child));
                    break;
                case Status:
                    contribution.setStatus(getString(child));
                    break;
                }
            }
        }
        return contribution;
    }

    public static Contribution createContribution(String contributionDocument) {
        return createContribution(getDocumentFromString(contributionDocument));
    }

}
