package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.List;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.model.Attribute;
import com.indicee.api.objects.model.Field;

public final class IndiceeModelParser extends IndiceeParser {

    private enum FieldChild {
        UUID("uuid"), Name("name"), QName("qname"), Attributes("attributes"), DataType("data_type"), AggregateType(
                "aggregate_type"), TrackingType("tracking_type"), Formula("formula");

        private final String value;

        private FieldChild(String value) {
            this.value = value;
        }

        public static FieldChild get(String name) {
            for (FieldChild fieldChild : FieldChild.values()) {
                if (fieldChild.value.equals(name)) {
                    return fieldChild;
                }
            }
            return null;
        }
    }

    public List<Field> getFields(Document model) {
        List<Field> indiceeFields = new ArrayList<Field>();
        NodeList fields = model.getElementsByTagName("fields").item(0).getChildNodes();
        for (int i = 0; i < fields.getLength(); i++) {
            Node field = fields.item(i);
            Field indiceeField = new Field();

            for (int j = 0; j < field.getChildNodes().getLength(); j++) {
                Node fieldValue = field.getChildNodes().item(j);
                FieldChild fieldChild = FieldChild.get(fieldValue.getLocalName());
                switch (fieldChild) {
                case UUID:
                    indiceeField.setUuid(fieldValue.getFirstChild().getNodeValue());
                    break;
                case Name:
                    indiceeField.setName(fieldValue.getFirstChild().getNodeValue());
                    break;
                case QName:
                    indiceeField.setQname(fieldValue.getFirstChild().getNodeValue());
                    break;
                case Attributes:
                    indiceeField.setAttributes(getAttributes(fieldValue.getChildNodes()));
                    break;
                case DataType:
                    indiceeField.setDataType(Field.DataType.get(fieldValue.getFirstChild().getNodeValue()));
                    break;
                case AggregateType:
                    indiceeField.setAggregateType(Field.AggregateType.get(fieldValue.getFirstChild().getNodeValue()));
                    break;
                case TrackingType:
                    indiceeField.setTrackingType(Field.TrackingType.get(fieldValue.getFirstChild().getNodeValue()));
                    break;
                case Formula:
                    indiceeField.setFormula(fieldValue.getFirstChild().getNodeValue());
                    break;
                }
            }
            indiceeFields.add(indiceeField);
        }
        return indiceeFields;
    }

    private List<Attribute> getAttributes(NodeList attributeNodes) {
        List<Attribute> attributes = new ArrayList<Attribute>();
        for (int i = 0; i < attributeNodes.getLength(); i++) {
            Attribute attribute = new Attribute();
            NamedNodeMap attributeNodeAttributes = attributeNodes.item(i).getAttributes();
            attribute.setName(attributeNodeAttributes.getNamedItem("name").getNodeValue());
            attribute.setType(Attribute.Type.get(attributeNodeAttributes.getNamedItem("type").getNodeValue()));
            attribute.setValue(attributeNodeAttributes.getNamedItem("value").getNodeValue());
            attributes.add(attribute);
        }
        return attributes;
    }

}
