package com.indicee.api.parser;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.indicee.api.objects.user.User;

public final class UserParser extends IndiceeParser {

    private enum UserElement {
        ID("id"), FirstName("first_name"), LastName("last_name"), Company("company"), DisplayName("display_name"), Email(
                "email"), Industry("industry"), Phone("phone"), Status("status"), Title("title"), Description(
                "description"), Password("password");

        public static UserElement get(String name) {
            for (UserElement item : UserElement.values()) {
                if (item.value.equals(name)) {
                    return item;
                }
            }
            return null;
        }

        private final String value;

        private UserElement(String value) {
            this.value = value;
        }
    }

    public static User createUser(Document userDocument) {
        return createUser(userDocument.getDocumentElement());
    }

    private static User createUser(Node node) {
        User user = new User();
        for (int i = 0; i < node.getChildNodes().getLength(); i++) {
            Node child = node.getChildNodes().item(i);
            UserElement elem = UserElement.get(child.getNodeName());
            if (elem != null) {
                switch (elem) {
                case ID:
                    user.setId(getInteger(child));
                    break;
                case FirstName:
                    user.setFirstName(getString(child));
                    break;
                case LastName:
                    user.setLastName(getString(child));
                    break;
                case Company:
                    user.setCompany(getString(child));
                    break;
                case DisplayName:
                    user.setDisplayName(getString(child));
                    break;
                case Email:
                    user.setEmail(getString(child));
                    break;
                case Industry:
                    user.setIndustry(getString(child));
                    break;
                case Phone:
                    user.setPhone(getString(child));
                    break;
                case Status:
                    user.setStatus(getString(child));
                    break;
                case Title:
                    user.setTitle(getString(child));
                    break;
                case Description:
                    user.setDescription(getString(child));
                    break;
                case Password:
                    user.setPassword(getString(child));
                    break;
                }
            }
        }
        return user;
    }

    public static User createUser(String userDocument) {
        return createUser(getDocumentFromString(userDocument));
    }

    public static List<User> createUsers(Document usersDocument) {
        List<User> users = new ArrayList<User>();
        NodeList userNodes = usersDocument.getElementsByTagName("user");
        for (int i = 0; i < userNodes.getLength(); i++) {
            users.add(createUser(userNodes.item(i)));
        }
        return users;
    }

    public static List<User> createUsers(String usersDocument) {
        return createUsers(getDocumentFromString(usersDocument));
    }

    public static Map<String, String> getParameters(User user) {
        Map<String, String> parameter = new HashMap<String, String>();
        if (user.getFirstName() != null) {
            parameter.put(UserElement.FirstName.value, user.getFirstName());
        }
        if (user.getLastName() != null) {
            parameter.put(UserElement.LastName.value, user.getLastName());
        }
        if (user.getDisplayName() != null) {
            parameter.put(UserElement.DisplayName.value, user.getDisplayName());
        }
        if (user.getDescription() != null) {
            parameter.put(UserElement.Description.value, user.getDescription());
        }
        if (user.getIndustry() != null) {
            parameter.put(UserElement.Industry.value, user.getIndustry());
        }
        if (user.getCompany() != null) {
            parameter.put(UserElement.Company.value, user.getCompany());
        }
        if (user.getTitle() != null) {
            parameter.put(UserElement.Title.value, user.getTitle());
        }
        if (user.getPhone() != null) {
            parameter.put(UserElement.Phone.value, user.getPhone());
        }
        if (user.getEmail() != null) {
            parameter.put(UserElement.Email.value, user.getEmail());
        }
        if (user.getPassword() != null) {
            parameter.put(UserElement.Password.value, user.getPassword());
        }
        if (user.getStatus() != null) {
            parameter.put(UserElement.Status.value, user.getStatus());
        }
        return parameter;
    }
}
