package com.indicee.api.services;

import java.io.File;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.objects.contribution.Contribution;

/**
 * The Contribution service is used to upload contributions to Indicee.
 * 
 * @author victork
 * 
 */
public interface ContributionService {

    /**
     * This will upload the given file to Indicee. If the contribution is match
     * to an existing one it will automatically be added to that dataset.
     * 
     * @param file
     *            The contribution file to upload
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error. This error will also be thrown if
     *             there is an error reading the file.
     */
    public Contribution upload(File file) throws IndiceeAPIConnectorException;

    /**
     * This will upload the given file to Indicee. The contribution will be
     * added to the specified dataset. An error is returned it the signature of
     * the dataset does not match the signature of the contribution
     * 
     * @param file
     *            The contribution file to upload
     * @param datasetID
     *            The id of the dataset to add the contribution to.
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error. This error will also be thrown if
     *             there is an error reading the file.
     */
    public Contribution upload(File file, String datasetID) throws IndiceeAPIConnectorException;

    /**
     * This will upload the given stream to Indicee. If the contribution is
     * match to an existing one it will automatically be added to that dataset.
     * 
     * @param inputStreamWrapper
     *            The contribution stream to upload
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Contribution upload(InputStreamWrapper inputStreamWrapper) throws IndiceeAPIConnectorException;

    /**
     * This will upload the given file to Indicee. The contribution will be
     * added to the specified dataset. An error is returned it the signature of
     * the dataset does not match the signature of the contribution
     * 
     * @param inputStreamWrapper
     *            The contribution stream to upload
     * @param datasetID
     *            The id of the dataset to add the contribution to.
     * @return The contribution information
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Contribution upload(InputStreamWrapper inputStreamWrapper, String datasetID)
            throws IndiceeAPIConnectorException;

}
