package com.indicee.api.services;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.ViewWindow;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.view.View;
import com.indicee.api.objects.report.view.filter.Filter;

/**
 * The report service is use to create, and view reports from Indicee.
 * 
 * @author victork
 * 
 */
public interface ReportService {

    /**
     * Creates a report with the given name. The default {@link ViewWindow} is
     * used.
     * 
     * @param name
     *            The name of the report to use.
     * @return The report ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report createReport(String name) throws IndiceeAPIConnectorException;

    /**
     * Creates a report with the given name and view.
     * 
     * @param name
     *            The name of the report to use.
     * @param view
     *            The view for the report.
     * @return The report ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report createReport(String name, View view) throws IndiceeAPIConnectorException;

    /**
     * Deletes the report give the report id.
     * 
     * @param reportID
     *            The ID of the report to delete.
     * @return The report ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report deleteReport(int reportID) throws IndiceeAPIConnectorException;

    /**
     * Returns the report give the report id. The dedfault {@link ViewWindow} is
     * used.
     * 
     * @param reportID
     *            The ID of the report to use.
     * @return The report specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report getReport(int reportID) throws IndiceeAPIConnectorException;

    /**
     * Returns the report give the report id
     * 
     * @param reportID
     *            The ID of the report to use.
     * @param fieldFilters
     *            A list of field filters to add to the report.
     * @return The report specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report getReport(int reportID, List<Filter> fieldFilters) throws IndiceeAPIConnectorException;

    /**
     * Returns the report give the report id
     * 
     * @param reportID
     *            The ID of the report to use.
     * @param fieldFilters
     *            A list of field filters to add to the report.
     * @param viewWindow
     *            The view window for the data to return.
     * @return The report specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report getReport(int reportID, List<Filter> fieldFilters, ViewWindow viewWindow)
            throws IndiceeAPIConnectorException;

    /**
     * Returns the report give the report id
     * 
     * @param reportID
     *            The ID of the report to use.
     * @param viewWindow
     *            The view window for the data to return.
     * @return The report specified by the given ID.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public Report getReport(int reportID, ViewWindow viewWindow) throws IndiceeAPIConnectorException;

    /**
     * Returns the list of reports
     * 
     * @return The list of reports.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public List<Report> listReports() throws IndiceeAPIConnectorException;
}
