package com.indicee.api.services;

import java.util.List;

import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.user.User;

/**
 * The user service is use to create, update and view users from Indicee.
 * 
 * @author victork
 * 
 */
public interface UserService {

    /**
     * Creates a new user.
     * 
     * @param user
     *            The new user to create
     * @return A user object that only contains data for the new user id.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public User createUser(User user) throws IndiceeAPIConnectorException;

    /**
     * Gets the currenlt logged in user.
     * 
     * @return The currently logged in user.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public User getCurrentUser() throws IndiceeAPIConnectorException;

    /**
     * Gets the user with the given ID.
     * 
     * @param userID
     *            The id of the user to get.
     * @return The user with the given ID
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public User getUser(int userID) throws IndiceeAPIConnectorException;

    /**
     * Gets a list of users.
     * 
     * @param userIDs
     *            The ids of the users to get
     * @return The list of users
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public List<User> getUsers(int[] userIDs) throws IndiceeAPIConnectorException;

    /**
     * Updates an existing user.
     * 
     * @param user
     *            The users to update.
     * @return A user object that only contains data for the updated user id.
     * @throws IndiceeAPIConnectorException
     *             Thrown if there is a problem connecting to Indicee, or the
     *             server returned an error.
     */
    public User updateUser(User user) throws IndiceeAPIConnectorException;
}
