package com.indicee.api.services.impl;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.connector.util.InputStreamWrapper;
import com.indicee.api.objects.contribution.Contribution;
import com.indicee.api.parser.ContributionParser;
import com.indicee.api.services.ContributionService;
import com.indicee.api.services.IndiceeAPIService;

/**
 * This is the implementation of the {@link ContributionService}
 * 
 * @author victork
 * 
 */
public class ContributionServiceImpl extends IndiceeAPIService implements ContributionService {

    private static final String UPLOAD_CONTRIBUTION_URL = "/api/contributions/upload.xml";

    /**
     * Creates a new contribution service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public ContributionServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(File file) throws IndiceeAPIConnectorException {
        return upload(file, null);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(File file, String datasetID) throws IndiceeAPIConnectorException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("file", file);
        if (datasetID != null && !datasetID.isEmpty()) {
            parameters.put("dataset_uuid", datasetID);
        }
        Document contributionDocument = connector.makePostCallRtnDocument(UPLOAD_CONTRIBUTION_URL, parameters);
        return ContributionParser.createContribution(contributionDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(InputStreamWrapper inputStreamWrapper) throws IndiceeAPIConnectorException {
        return upload(inputStreamWrapper, null);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Contribution upload(InputStreamWrapper inputStreamWrapper, String datasetID)
            throws IndiceeAPIConnectorException {
        Map<String, Object> parameters = new HashMap<String, Object>();
        parameters.put("file", inputStreamWrapper);
        if (datasetID != null && !datasetID.isEmpty()) {
            parameters.put("dataset_uuid", datasetID);
        }
        Document contributionDocument = connector.makePostCallRtnDocument(UPLOAD_CONTRIBUTION_URL, parameters);
        return ContributionParser.createContribution(contributionDocument);
    }
}
