package com.indicee.api.services.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.ViewWindow;
import com.indicee.api.objects.report.Report;
import com.indicee.api.objects.report.view.View;
import com.indicee.api.objects.report.view.filter.Filter;
import com.indicee.api.parser.ReportParser;
import com.indicee.api.services.IndiceeAPIService;
import com.indicee.api.services.ReportService;

/**
 * This is the implementation of the {@link ReportService}
 * 
 * @author victork
 * 
 */
public class ReportServiceImpl extends IndiceeAPIService implements ReportService {

    private static final String REPORT_URL = "/api/reports";
    private static final String REPORT_VIEW_URL = "view";
    private static final String REPORT_CREATE_URL = "create";
    private static final String REPORT_DELETE_URL = "delete";

    /**
     * Creates a new report service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public ReportServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Report getReport(int reportID) throws IndiceeAPIConnectorException {
        return getReport(reportID, new ViewWindow());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Report getReport(int reportID, List<Filter> fieldFilters) throws IndiceeAPIConnectorException {
        return getReport(reportID, fieldFilters, new ViewWindow());
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Report getReport(int reportID, List<Filter> fieldFilters, ViewWindow viewWindow)
            throws IndiceeAPIConnectorException {
        Document reportViewDocument = connector.makeGetCallRtnDocument(REPORT_URL + URL_SLASH + reportID + URL_SLASH
                + REPORT_VIEW_URL);
        View reportView = ReportParser.createReportView(reportViewDocument);
        reportView.getFieldFilters().addAll(fieldFilters);
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("view", reportView.toFormatedString());
        params.put("window", viewWindow);
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + URL_SLASH + reportID, params);
        return ReportParser.createReport(reportDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public Report getReport(int reportID, ViewWindow viewWindow) throws IndiceeAPIConnectorException {
        Document reportDocument = connector.makeGetCallRtnDocument(REPORT_URL + URL_SLASH + reportID
                + URL_QUERY_PARAM_START + "window=" + viewWindow);
        return ReportParser.createReport(reportDocument);
    }

    @Override
    public Report createReport(String name) throws IndiceeAPIConnectorException {
        return createReport(name, null);
    }

    @Override
    public Report createReport(String name, View view) throws IndiceeAPIConnectorException {
        Map<String, Object> params = new HashMap<String, Object>();
        params.put("name", name);
        if (view != null) {
            params.put("view", view.toFormatedString());
        }
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + URL_SLASH + REPORT_CREATE_URL, params);
        return ReportParser.createReport(reportDocument);
    }

    @Override
    public Report deleteReport(int reportID) throws IndiceeAPIConnectorException {
        Document reportDocument = connector.makePostCallRtnDocument(REPORT_URL + URL_SLASH + reportID + URL_SLASH
                + REPORT_DELETE_URL);
        return ReportParser.createReport(reportDocument);
    }

    @Override
    public List<Report> listReports() throws IndiceeAPIConnectorException {
        Document reportDocument = connector.makeGetCallRtnDocument(REPORT_URL);
        return ReportParser.createReports(reportDocument);
    }
}
