package com.indicee.api.services.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;

import org.w3c.dom.Document;

import com.indicee.api.connector.IndiceeAPIConnector;
import com.indicee.api.connector.exception.IndiceeAPIConnectorException;
import com.indicee.api.objects.user.User;
import com.indicee.api.parser.UserParser;
import com.indicee.api.services.IndiceeAPIService;
import com.indicee.api.services.UserService;

/**
 * This is the implementation of the {@link UserService}
 * 
 * @author victork
 * 
 */
public class UserServiceImpl extends IndiceeAPIService implements UserService {

    private static final String GET_USER_URL = "/api/users/show.xml?user_ids=";
    private static final String CREATE_USER_URL = "/api/users/create.xml";

    /**
     * Creates a new user service.
     * 
     * @param connector
     *            The {@link IndiceeAPIConnector} to use.
     */
    public UserServiceImpl(IndiceeAPIConnector connector) {
        super(connector);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public User createUser(User user) throws IndiceeAPIConnectorException {
        Map<String, String> parameters = UserParser.getParameters(user);
        Document userDocument = connector.makePostCallRtnDocument(CREATE_USER_URL, parameters);
        return UserParser.createUser(userDocument);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public User getCurrentUser() throws IndiceeAPIConnectorException {
        return getUsers(new int[] {}).get(0);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public User getUser(int userID) throws IndiceeAPIConnectorException {
        List<User> users = getUsers(new int[] { userID });
        if (users.isEmpty()) {
            throw new IndiceeAPIConnectorException("Error getting user: User " + userID + " cannot be found.");
        }
        return getUsers(new int[] { userID }).get(0);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public List<User> getUsers(int userIDs[]) throws IndiceeAPIConnectorException {
        String userIDsParam = "";
        List<User> otherUsers = null;
        for (int i = 0; i < userIDs.length; i++) {
            if (i == 100) {
                otherUsers = new ArrayList<User>(getUsers(Arrays.copyOfRange(userIDs, 100, userIDs.length)));
                break;
            }
            userIDsParam += userIDs[i] + ",";
        }
        if (!userIDsParam.isEmpty()) {
            userIDsParam = userIDsParam.substring(0, userIDsParam.length() - 1);
        }
        Document userDocument = connector.makeGetCallRtnDocument(GET_USER_URL + userIDsParam);
        List<User> users = UserParser.createUsers(userDocument);
        if (otherUsers != null) {
            users.addAll(otherUsers);
        }
        return users;
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public User updateUser(User user) throws IndiceeAPIConnectorException {
        Map<String, String> parameters = UserParser.getParameters(user);
        Document userDocument = connector.makePostCallRtnDocument(CREATE_USER_URL, parameters);
        return UserParser.createUser(userDocument);
    }
}
